import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Represents a brick of a piece
 * 
 * @author Alexandre Brabant (National Education France)
 * @version February 2012
 */
public class Brick extends Actor
{
    private static final int RED = 0;
    private static final int YELLOW = 1;
    private static final int GREEN = 2;
    private static final int BLUE = 3;

    private int color;
    private boolean removable;
    
    public Brick()
    {
        removable = false;
        // The color of the brick is randomly selected
        color = Greenfoot.getRandomNumber(4);
        switch(color) {
            case RED :
                setImage("brick-red.png");
                break;
            case YELLOW :
                setImage("brick-yellow.png");
                break;
            case GREEN :
                setImage("brick-green.png");
                break;
            case BLUE :
                setImage("brick-blue.png");
                break;
        }
    }
    /**
     * return the color of the brick
     */
    public int getColor()
    {
        return color;
    }
    /**
     * set the color of the brick
     */
    public void setColor(int theColor)
    {
        color = theColor;
    }
    /**
     * 
     */
    private void setRemovable(boolean isRemovable)
    {
        removable = isRemovable;
        if (removable) scratch();
    }
    /**
     * 
     */
    protected boolean isRemovable()
    {
        return removable;
    }
    /**
     * 
     */
    private boolean tryVerticalTetrisMoins()
    {
        boolean response = false;
        // OFFSET 1
        if (this.getY()<getWorld().getHeight()-2 && this.getOneObjectAtOffset(0,1,Actor.class) != null && ((Brick)this.getOneObjectAtOffset(0,1,Actor.class)).getColor() == this.color)
        {
             // OFFSET 2
             if (this.getY()<getWorld().getHeight()-3 && this.getOneObjectAtOffset(0,2,Actor.class) != null && ((Brick)this.getOneObjectAtOffset(0,2,Actor.class)).getColor() == this.color)
             {
                 this.setRemovable(true);
                 ((Brick)this.getOneObjectAtOffset(0,1,Actor.class)).setRemovable(true);
                 ((Brick)this.getOneObjectAtOffset(0,2,Actor.class)).setRemovable(true);
                 ((TetrisMoinsWorld)getWorld()).addPoints(15);
                 response = true;
             }
        }
        return response;
    }
    /**
     * 
     */
    private boolean tryHorizontalTetrisMoins()
    {
        boolean response = false;
        // OFFSET 1
        if (this.getOneObjectAtOffset(1,0,Actor.class) != null && ((Brick)this.getOneObjectAtOffset(1,0,Actor.class)).getColor() == this.color)
        {
            // OFFSET 2
            if (this.getOneObjectAtOffset(2,0,Actor.class) != null && ((Brick)this.getOneObjectAtOffset(2,0,Actor.class)).getColor() == this.color)
            {
                // OFFSET -1
                if (this.getOneObjectAtOffset(-1,0,Actor.class) != null && ((Brick)this.getOneObjectAtOffset(-1,0,Actor.class)).getColor() == this.color)
                {
                    // OFFSET -2
                    if (this.getOneObjectAtOffset(-2,0,Actor.class) != null && ((Brick)this.getOneObjectAtOffset(-2,0,Actor.class)).getColor() == this.color)
                    {
                        ((Brick)this.getOneObjectAtOffset(-2,0,Actor.class)).setRemovable(true);
                        ((Brick)this.getOneObjectAtOffset(-1,0,Actor.class)).setRemovable(true);
                        this.setRemovable(true);
                        ((Brick)this.getOneObjectAtOffset(1,0,Actor.class)).setRemovable(true);
                        ((Brick)this.getOneObjectAtOffset(2,0,Actor.class)).setRemovable(true);
                        ((TetrisMoinsWorld)getWorld()).addPoints(50);
                        response = true;
                    }
                    else
                    {
                        ((Brick)this.getOneObjectAtOffset(-1,0,Actor.class)).setRemovable(true);
                        this.setRemovable(true);
                        ((Brick)this.getOneObjectAtOffset(1,0,Actor.class)).setRemovable(true);
                        ((Brick)this.getOneObjectAtOffset(2,0,Actor.class)).setRemovable(true);
                        ((TetrisMoinsWorld)getWorld()).addPoints(30);
                        response = true;
                    }
                }
                else
                {
                    this.setRemovable(true);
                    ((Brick)this.getOneObjectAtOffset(1,0,Actor.class)).setRemovable(true);
                    ((Brick)this.getOneObjectAtOffset(2,0,Actor.class)).setRemovable(true);
                    ((TetrisMoinsWorld)getWorld()).addPoints(15);
                    response = true;
                }
            }    // OFFSET -1
            else if (this.getOneObjectAtOffset(-1,0,Actor.class) != null && ((Brick)this.getOneObjectAtOffset(-1,0,Actor.class)).getColor() == this.color)
            {
                // OFFSET -2
                if (this.getOneObjectAtOffset(-2,0,Actor.class) != null && ((Brick)this.getOneObjectAtOffset(-2,0,Actor.class)).getColor() == this.color)
                {
                    ((Brick)this.getOneObjectAtOffset(-2,0,Actor.class)).setRemovable(true);
                    ((Brick)this.getOneObjectAtOffset(-1,0,Actor.class)).setRemovable(true);
                    this.setRemovable(true);
                    ((Brick)this.getOneObjectAtOffset(1,0,Actor.class)).setRemovable(true);
                    ((TetrisMoinsWorld)getWorld()).addPoints(30);
                    response = true;
                }
                else
                {
                    ((Brick)this.getOneObjectAtOffset(-1,0,Actor.class)).setRemovable(true);
                    this.setRemovable(true);
                    ((Brick)this.getOneObjectAtOffset(1,0,Actor.class)).setRemovable(true);
                    ((TetrisMoinsWorld)getWorld()).addPoints(15);
                    response = true;
                }
            }
        }    // OFFSET -1
        else if (this.getOneObjectAtOffset(-1,0,Actor.class) != null && ((Brick)this.getOneObjectAtOffset(-1,0,Actor.class)).getColor() == this.color)
        {
            // OFFSET -2
            if (this.getOneObjectAtOffset(-2,0,Actor.class) != null && ((Brick)this.getOneObjectAtOffset(-2,0,Actor.class)).getColor() == this.color)
            {
                ((Brick)this.getOneObjectAtOffset(-2,0,Actor.class)).setRemovable(true);
                ((Brick)this.getOneObjectAtOffset(-1,0,Actor.class)).setRemovable(true);
                this.setRemovable(true);
                ((TetrisMoinsWorld)getWorld()).addPoints(15);
                response = true;
            }
        }
        return response;
    }
    /**
     * 
     */
    protected boolean tryTetrisMoins(boolean horizontaly, boolean verticaly)
    {
        if (horizontaly) horizontaly = tryHorizontalTetrisMoins();
        if (verticaly) verticaly = tryVerticalTetrisMoins(); 
        return (horizontaly || verticaly);
    }
    /**
     * 
     */
    private void scratch()
    {
        switch(color) {
            case RED :
                setImage("brick-red&black.png");
                break;
            case YELLOW :
                setImage("brick-yellow&black.png");
                break;
            case GREEN :
                setImage("brick-green&black.png");
                break;
            case BLUE :
                setImage("brick-blue&black.png");
                break;
        }
    }
}
