import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Represents a piece; contains the act-method
 * 
 * @author Alexandre Brabant (National Education, France)
 * @version February 2012
 */
public class Piece extends Actor
{
    private Brick[] brick; // a piece consists of two brick
    private static final int NORMAL = 0;
    private static final int TETRISMOINS_FOUND = 1;
    private static final int TETRISMOINS_REMOVED = 2;
    private static final int BRICKS_WENTDOWN = 3;
    private int status = NORMAL;
    /**
     * 
     */
    public Piece()
    {
        setImage("ground.png");
        brick = new Brick[2];
        brick[0] = new Brick();
        brick[1] = new Brick();
    }
    /**
     * 
     */
    protected void addedToWorld(World world)
    {
        getWorld().addObject(brick[0], 5, 0);
        getWorld().addObject(brick[1], 5, 1);
    }
    /**
     * Act - do whatever the Piece wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        switch (status)
        {
            case NORMAL :
                if (Greenfoot.isKeyDown("right") && canMoveRight()) moveRight();
                else if (Greenfoot.isKeyDown("left") && canMoveLeft()) moveLeft();
                else if (Greenfoot.isKeyDown("up")) shift();
                else if (Greenfoot.isKeyDown("down")) while (canFall()) fall();
                else if (canFall()) fall();
                else if (tryTetrisMoins()) status = TETRISMOINS_FOUND;
                else 
                {
                    if (((TetrisMoinsWorld)getWorld()).isGameOver()) ((TetrisMoinsWorld)getWorld()).gameOver();
                    else ((TetrisMoinsWorld)getWorld()).addNewPiece();
                    remove();
                }
                break;
            case TETRISMOINS_FOUND :
                ((TetrisMoinsWorld)getWorld()).removeTetrisMoinsBricks();
                status = TETRISMOINS_REMOVED;
                break;
            case TETRISMOINS_REMOVED :
                if (((TetrisMoinsWorld)getWorld()).tryToGoDownBricks()) status = BRICKS_WENTDOWN;
                else
                {
                    if (((TetrisMoinsWorld)getWorld()).isGameOver()) ((TetrisMoinsWorld)getWorld()).gameOver();
                    else ((TetrisMoinsWorld)getWorld()).addNewPiece();
                    status = NORMAL;
                    remove();
                }
                break;
            case BRICKS_WENTDOWN :
                if (((TetrisMoinsWorld)getWorld()).tryTetrisMoins()) status = TETRISMOINS_FOUND;
                else
                {
                    if (((TetrisMoinsWorld)getWorld()).isGameOver()) ((TetrisMoinsWorld)getWorld()).gameOver();
                    else ((TetrisMoinsWorld)getWorld()).addNewPiece();
                    status = NORMAL;
                    remove();
                }
                break;            
        }
    }
    /**
     * Test if it can move one column to the right
     */
    private boolean canMoveRight()
    {
        if (getWorld().getObjectsAt(brick[1].getX()+1,brick[1].getY(),Actor.class).size() > 0) return false;
        return true;
    }
    /**
     * Move one column to the right
     */
    private void moveRight()
    {
        brick[0].setLocation(brick[0].getX() + 1,brick[0].getY());
        brick[1].setLocation(brick[1].getX() + 1,brick[1].getY());
    }
    /**
     * Test if it can move one column to the left
     */
    private boolean canMoveLeft()
    {
        if (getWorld().getObjectsAt(brick[1].getX()-1,brick[1].getY(),Actor.class).size() > 0) return false;
        return true;
    }
    /**
     * Move one column to the left
     */
    private void moveLeft()
    {
        brick[0].setLocation(brick[0].getX() - 1,brick[0].getY());
        brick[1].setLocation(brick[1].getX() - 1,brick[1].getY());
    }
    /**
     * 
     */
    private void shift()
    {
        GreenfootImage image = brick[0].getImage();
        int color = brick[0].getColor();
        brick[0].setImage(brick[1].getImage());
        brick[0].setColor(brick[1].getColor());
        brick[1].setImage(image);
        brick[1].setColor(color);
    }
    /**
     * Test if it can slide one row down
     */
    protected boolean canFall()
    {
        if (getWorld().getObjectsAt(brick[1].getX(),brick[1].getY()+1,Actor.class).size() > 0) return false;
        return true;
    }
    /**
     * Slide one row down
     */
    protected void fall()
    {
        brick[0].setLocation(brick[0].getX(),brick[0].getY()+1);
        brick[1].setLocation(brick[1].getX(),brick[1].getY()+1);
    }
    /**
     * 
     */
    private void remove()
    {
        TetrisMoinsWorld world = (TetrisMoinsWorld)getWorld();
        world.removeObject(this);
    }
    /**
     * 
     */
    private boolean tryTetrisMoins()
    {
        boolean b0 = brick[0].tryTetrisMoins(true,true);
        boolean b1 = brick[1].tryTetrisMoins(true,true);
        return (b0 || b1);
    }
}
