import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Write a description of class TetrisMoinsWorld here.
 * 
 * @author Alexandre Brabant (National Education, France)
 * @version February 2012
 */
public class TetrisMoinsWorld extends World
{
    private static final int WORLD_WIDTH = 9;
    private static final int WORLD_HEIGHT = 19;
    private static final int CELL_SIZE = 20;
    private static final int INIT_SPEED = 30;
    private int speed = INIT_SPEED;
    private int score = 0;
    private ScoreBoard scoreBoard;

    /**
     * Create a new world with WORLD_WIDTH x WORLD_HEIGHT cells and
     * with a cell size of CELL_SIZE x CELL_SIZE pixels
     */
    public TetrisMoinsWorld()
    {
        super(WORLD_WIDTH, WORLD_HEIGHT, CELL_SIZE);
        setBackground("cell.png");
        for (int i=0;i<WORLD_WIDTH;i++) addObject(new Ground(),i,WORLD_HEIGHT-1);
        Greenfoot.setSpeed(speed);
        scoreBoard = new ScoreBoard();
        java.awt.Color color = new java.awt.Color(255,255,255,0);
        GreenfootImage image = new GreenfootImage("SCORE : "+score,16,java.awt.Color.black,color);
        scoreBoard.setImage(image);
        addObject(scoreBoard,4,WORLD_HEIGHT-1);
        addNewPiece();
    }
    /**
     * Create a new piece and add it to the world
     */
    protected void addNewPiece()
    {
        Piece piece = new Piece();
        addObject(piece,0,WORLD_HEIGHT-1);
    }
    /**
     * 
     */
    boolean isGameOver()
    {
        if (getObjectsAt(5,0,Brick.class).size() > 0) return true;
        return false;
    }
    /**
     * Game over
     */
    protected void gameOver()
    {
        ScoreBoard endBoard = new ScoreBoard();
        java.awt.Color color = new java.awt.Color(255,255,255,127);
        GreenfootImage image = new GreenfootImage("END",64,java.awt.Color.black,color);
        endBoard.setImage(image);
        addObject(endBoard,4,8);
        Greenfoot.stop();
    }
    /**
     * 
     */
    protected void removeTetrisMoinsBricks()
    {
        for (int i=0;i<WORLD_WIDTH;i++)
        {
            for (int j=0;j<WORLD_HEIGHT-1;j++)
            {
                java.util.List bricks = getObjectsAt(i,j,Brick.class);
                if (bricks.size()>0 && ((Brick)bricks.get(0)).isRemovable())
                    removeObject((Brick)bricks.get(0));
            }
        }
    }
    /**
     * 
     */
    protected boolean tryToGoDownBricks()
    {
        boolean response = false;
        for (int i=0;i<WORLD_WIDTH;i++)
        {
            for (int j=WORLD_HEIGHT-3;j>1;j--)
            {
                java.util.List bricks = getObjectsAt(i,j,Brick.class);
                if (bricks.size()>0)
                {
                    Brick b = (Brick)bricks.get(0);
                    int k = 1;
                    while (getObjectsAt(i,j+k,Brick.class).size()==0 && j+k <WORLD_HEIGHT-1)
                    {
                        b.setLocation(i,b.getY()+1);
                        k++;
                        response = true;
                    }
                }
            }
        }
        return response;
    }
    /**
     * 
     */
    protected boolean tryTetrisMoins()
    {
        boolean response = false;
        for (int j=0;j<WORLD_HEIGHT-1;j++)
        {
            int c = 0;
            for (int i=0;i<=9;i++)
            {
                java.util.List bricks = getObjectsAt(i,j,Brick.class);
                if (bricks.size()>0)
                {
                    if (c == 0 || c == 3 || c == 6)
                    {
                        if (response) {boolean b = ((Brick)bricks.get(0)).tryTetrisMoins(true,true);}
                        else response = ((Brick)bricks.get(0)).tryTetrisMoins(true,true);
                    }
                    else
                    {
                        if (response) {boolean b = ((Brick)bricks.get(0)).tryTetrisMoins(false,true);}
                        else response = ((Brick)bricks.get(0)).tryTetrisMoins(false,true);
                    }
                }
                c++;
            }
        }
        return response;
    }
    /**
     * 
     */
    protected void addPoints(int points)
    {
        score+=points;
        java.awt.Color color = new java.awt.Color(255,255,255,0);
        GreenfootImage image = new GreenfootImage("SCORE : "+score,16,java.awt.Color.black,color);
        scoreBoard.setImage(image);
        tryToIncreaseSpeed();
    }
    /**
     * 
     */
    private void tryToIncreaseSpeed()
    {
        if (score > 100 && speed == INIT_SPEED) Greenfoot.setSpeed(speed=INIT_SPEED+1);
        if (score > 200 && speed == INIT_SPEED+1) Greenfoot.setSpeed(speed=INIT_SPEED+2);
        if (score > 300 && speed == INIT_SPEED+2) Greenfoot.setSpeed(speed=INIT_SPEED+3);
        if (score > 400 && speed == INIT_SPEED+3) Greenfoot.setSpeed(speed=INIT_SPEED+4);
        if (score > 500 && speed == INIT_SPEED+4) Greenfoot.setSpeed(speed=INIT_SPEED+5);
        if (score > 600 && speed == INIT_SPEED+5) Greenfoot.setSpeed(speed=INIT_SPEED+6);
        if (score > 700 && speed == INIT_SPEED+6) Greenfoot.setSpeed(speed=INIT_SPEED+7);
        if (score > 800 && speed == INIT_SPEED+7) Greenfoot.setSpeed(speed=INIT_SPEED+8);
        if (score > 900 && speed == INIT_SPEED+8) Greenfoot.setSpeed(speed=INIT_SPEED+9);
        if (score > 1000 && speed == INIT_SPEED+9) Greenfoot.setSpeed(speed=INIT_SPEED+10);
        if (score > 1100 && speed == INIT_SPEED+10) Greenfoot.setSpeed(speed=INIT_SPEED+11);
        if (score > 1200 && speed == INIT_SPEED+11) Greenfoot.setSpeed(speed=INIT_SPEED+12);
        if (score > 1300 && speed == INIT_SPEED+12) Greenfoot.setSpeed(speed=INIT_SPEED+13);
        if (score > 1400 && speed == INIT_SPEED+13) Greenfoot.setSpeed(speed=INIT_SPEED+14);
        if (score > 1500 && speed == INIT_SPEED+14) Greenfoot.setSpeed(speed=INIT_SPEED+15);
        if (score > 1600 && speed == INIT_SPEED+15) Greenfoot.setSpeed(speed=INIT_SPEED+16);
        if (score > 1700 && speed == INIT_SPEED+16) Greenfoot.setSpeed(speed=INIT_SPEED+17);
        if (score > 1800 && speed == INIT_SPEED+17) Greenfoot.setSpeed(speed=INIT_SPEED+18);
        if (score > 1900 && speed == INIT_SPEED+18) Greenfoot.setSpeed(speed=INIT_SPEED+19);
        if (score > 2000 && speed == INIT_SPEED+19) Greenfoot.setSpeed(speed=INIT_SPEED+10);
    }
}
